#!/usr/bin/python
# -*- encoding: utf-8 -*-

# testovaci program cteni maticove klavesnice na portu B expanderu s adresou 0x20

import smbus
import time

bus = smbus.SMBus(1)  # nova verze RasPi (512MB RAM) 
#bus = smbus.SMBus(0)   # stara verze RasPi (256MB RAM)

adr_exp = 0x20    # adresa expanderu s pripojenou klavesnici

# definice smeru signalu na expanderu pro klavesnici (vstupy/vystupy)
bus.write_byte_data(adr_exp,0x01,0b00001111) # B-port: 7,6,5,4 = radky (OUT) ;  3,2,1,0 = sloupce (IN)  
bus.write_byte_data(adr_exp,0x0D,0b00001111) # na vstupni bity portu B pripojit vnitrni Pull-Upy



#==============================================================================
# hlavni smycka pro skenovani maticove klavesnice 4*4 tlacitka
def cti_klavesnici():
  radka = ['    ' , '123A' , '456B' , '789C' , '*0#D']   # definice usporadani klavesnice

  rindex = 0        # index radky - v prubehu skenovani se meni od 1 do 4
  vystup = ""
  shift = False     # nektera z klaves muze mit nastavenou funkci "Shift"

  # do radek klavesnice (do hornich 4 bitu) se postupne posila "0"  (zacina se horni radkou - tlacitka [1][2][3][A])
  for r_bin  in [ 0b11100000 , 0b11010000 , 0b10110000 , 0b01110000 ]:  
    rindex = rindex + 1

    bus.write_byte_data(adr_exp , 0x13 , r_bin)   # zapsani "0" na patricnou radku
    time.sleep(0.01)

    sloupce =  bus.read_byte_data(adr_exp,0x13)
   
    if ( (sloupce & 8 ) == 0):                 # ctu pozici 0000 1000 (pravy sloupec klaves [A][B][C][D]) 
      vystup = vystup + radka[rindex][3:4] 

    if ( (sloupce & 4 ) == 0):                 # ctu pozici 0000 0100
      vystup = vystup + radka[rindex][2:3] 

    if ( (sloupce & 2 ) == 0):                 # ctu pozici 0000 0010 
      vystup = vystup + radka[rindex][1:2] 

    if ( (sloupce & 1 ) == 0):                 # ctu pozici 0000 0001 (levy sloupec klaves [1][4][7][*])
      vystup = vystup + radka[rindex][0:1] 
      if (radka[rindex][0:1] == "*"):           # klavesa [*] slouzi jako Shift
        shift = True
        

  # vystupem je kombinace stisknutych znaku a informace, jesti je mezi stisknutymi znaky take klavesa Shift [*]
  return vystup , shift
#==============================================================================


# podprogram, ktery preklada klavesy, stisknute spolecne se shiftem
# vstupem je napriklad kombinace klaves "6*" a vystupem je znak "k"
def preklad (kombinace):
  bez_shiftu = "123A456B789C*0#D"   # original bez shiftu
  se_shiftem = "efghijklmnop*qrs"   # preklad znaku pri pouziti Shiftu ([*] je Shift, tak se nepreklada)
  
  # v promenne "kombinace" muze byt nekolik znaku zaroven (kody stisknutych klaves a hvezdicka)
  # proto se musi vybrat jen prvni z techto znaku
  # vzhledem k tomu, ze se klavesa "*" vyhodnocuje pri skenu klavesnice az jako posledni,
  #    bude vzdycky na konci retezce.
  # Takze v promenne "znak" bude bud "*" pro pripad, ze bude stisknuty pouze Shift (a ten se nepreklada),
  # nebo bude v promenne "znak" prvni znak vstupniho retezce (klavesa stisknuta spolecne se Shiftem) 
  znak = kombinace[0:1]   
    
  # vlastni preklad znaku z promenne "bez_shiftu" na znak v promenne "se_shiftem"
  for i in range (16):
    if (bez_shiftu[i:i+1] == znak):
      prelozeno = se_shiftem[i:i+1]

  return (prelozeno)

#==============================================================================





#==============================================================================
# Hlavni smycka na vyhodnoceni "Shiftu" (klavesa [*])

slovo = ""
tlacitko = cti_klavesnici()

while slovo[len(slovo)-1:len(slovo)] != "D":   # tlacitko "D" ukoncuje zadavani (testovani posledniho znaku v 'slovo')
  while tlacitko[0] == "" or tlacitko[0] == "*" :  # kdyz neni stisknuto zadne tlacitko, nebo kdyz je stisknuty pouze Shift...
    tlacitko = cti_klavesnici()                    #  ... tak porad skenuje klavesnici 

  if (tlacitko[1] == True): # kdyz byl stisknuty Shift [*]
    znak = preklad(tlacitko[0])    # tak se kombinace tlacitek (napriklad 6* ) prelozi
  else:
    znak = tlacitko[0]             # kdyz Shift stisknuty nebyl, vlozi se do promenne "znak" jen stisknute tlacitko

  if (znak == "*"):    # [*] ma vyznam Shiftu, tak se samostatne nezobrazuje
    znak = ""
  else:
    print znak

  slovo = slovo + znak   # poskladani vsech tlacitek do jednoho retezce (posledni znak bude "D")
  while (tlacitko[0] != "" and tlacitko[0] != "*"):  # dokud neni tlacitko nebo Shift uvolnen, tak ceka
    tlacitko = cti_klavesnici()

print "vsechny stisknute klavesy: " + slovo
#==============================================================================


